@@ -8,6 +8,7 @@ from .errors import *
 from .field import Field
 from .util import (
     RecordingDict, DotNotationMixin, valid_client, NanomongoSONManipulator,
+    check_spec,
 )
 
 
@@ -25,15 +26,6 @@ class Index(object):
         self.args = args
         self.kwargs = kwargs
 
-    def ensure(self, collection, **kwargs):
-        """``ensure_index`` this Index on given collection"""
-        new_kwargs = {}
-        for k, v in self.kwargs.items():
-            new_kwargs[k] = v
-        for k, v in kwargs.items():
-            new_kwargs[k] = v
-        return collection.ensure_index(*self.args, **new_kwargs)
-
 
 class Nanomongo(object):
     """Contains information about the Document it's attached to like
@@ -122,7 +114,7 @@ class Nanomongo(object):
         """
         self.set_client(client) if client else None
         self.set_db(db_string) if db_string else None
-        self.set_collection() if collection else None
+        self.set_collection(collection) if collection else None
         self.check_config()
         self.add_son_manipulator()
         # indexes
@@ -203,16 +195,22 @@ class DocumentMeta(type):
 
     def check_index(cls, index):
         """check correctness of :class:`~Index` definitions"""
+        def valid_index_key(ikey):
+            if '.' not in ikey:
+                return cls.nanomongo.has_field(ikey)
+            field = ikey.split('.')[0]
+            return (cls.nanomongo.has_field(field) and
+                    cls.nanomongo.fields[field].data_type in [dict, list])
         i = index.args[0]  # key or list
         if not isinstance(i, (str, list)):
             raise TypeError('Index: str or list of key-value tuples expected')
-        if isinstance(i, str) and not cls.nanomongo.has_field(i):
+        if isinstance(i, str) and not valid_index_key(i):
             raise IndexMismatchError('field for index "%s" does not exist' % i)
         elif isinstance(i, list):
             for tup in i:
                 if not isinstance(tup, tuple) or 2 != len(tup):
                     raise TypeError('Index: list of key-value tuples expected')
-                if not cls.nanomongo.has_field(tup[0]):
+                if not valid_index_key(tup[0]):
                     err_str = 'field for index "%s" does not exist' % tup[0]
                     raise IndexMismatchError(err_str)
 
@@ -299,11 +297,15 @@ your document class with client, db, collection.''' % cls
     @classmethod
     def find(cls, *args, **kwargs):
         """``pymongo.Collection().find`` wrapper for this document"""
+        if args:
+            check_spec(cls, args[0])
         return cls.get_collection().find(*args, **kwargs)
 
     @classmethod
     def find_one(cls, *args, **kwargs):
         """``pymongo.Collection().find_one`` wrapper for this document"""
+        if args:
+            check_spec(cls, args[0])
         return cls.get_collection().find_one(*args, **kwargs)
 
     def __dir__(self):
@@ -383,7 +385,13 @@ your document class with client, db, collection.''' % cls
         diff['$set'].update(subdiff['$set'])
         diff['$unset'].update(subdiff['$unset'])
         diff['$addToSet'].update(subdiff['$addToSet'])
-        if not diff['$set'] and not diff['$unset'] and not diff['$addToSet']:
+        # remove empty update ops, MongoDB 2.6 returns error for them
+        diff = dict(filter(lambda update: update[1], diff.items()))
+        # for operation in ('$set', '$unset', '$addToSet'):
+        #     if {} == diff[operation]:
+        #         del diff[operation]
+        if not diff:
+            self.reset_diff()
             return
         response = self.get_collection().update(query, diff, **kwargs)
         self.reset_diff()
@@ -434,10 +442,12 @@ your document class with client, db, collection.''' % cls
             if ((self.nanomongo.has_field(field) and
                  list == self.nanomongo.fields[field].data_type)):
                 top_level_add(self, field, value)  # add & record
-            else:
+            elif self.nanomongo.has_field(field):
                 err_str = 'Cannot apply $addToSet modifier to non-array: %s=%s'
                 err_str = err_str % (field, self.nanomongo.fields[field].data_type)
                 raise ValidationError(err_str)
+            else:
+                raise ValidationError('Undefined field: "%s"' % top_key)
         # if deep-level
         else:
             try:
