@@ -208,6 +208,7 @@ class ClientTestCase(unittest.TestCase):
         m_count = len(Doc.get_collection().database.outgoing_copying_manipulators)
         self.assertEqual(1, m_count)
 
+        Doc2.register(client=client, db='nanotestdb', collection='doc2_collection')
 
 class MongoDocumentTestCase(unittest.TestCase):
     def setUp(self):
@@ -433,14 +434,24 @@ class IndexTestCase(unittest.TestCase):
                     Index([('foo', pymongo.ASCENDING), ('bar', pymongo.DESCENDING)]),
                 ]
 
-        count = 0
+        def def_mismatch_4():
+            class Doc(BaseDocument):
+                foo = Field(str)
+                bar = Field(int)
+                __indexes__ = [
+                    Index('foo.fail'),
+                    Index([('foo.fail', pymongo.ASCENDING), ('bar.fail', pymongo.ASCENDING)]),
+                ]
+
+        counts = {'type': 0, 'mismatch': 0}
         for fname, func in locals().items():  # run the above defined functions
-            count += 1
             if 'type' in fname:
+                counts['type'] += 1
                 self.assertRaises(TypeError, func)
             elif 'mismatch' in fname:
+                counts['mismatch'] += 1
                 self.assertRaises(IndexMismatchError, func)
-        self.assertTrue(7 <= count)
+        self.assertTrue(4 == counts['type'] == counts['mismatch'])
 
     @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
     def test_index_definitions(self):
@@ -471,6 +482,21 @@ class IndexTestCase(unittest.TestCase):
             __indexes__ = [Index('foo')]
 
         self.assertTrue(hasattr(Doc3, '__indexes__'))
-        Doc3.register(client, db='nanotestdb')
+        Doc3.register(client=client, db='nanotestdb')
         self.assertEqual(2, len(Doc3.get_collection().index_information()))  # 1 + _id
         self.assertFalse(hasattr(Doc3, '__indexes__'))
+
+        class Doc4(BaseDocument):
+            # test indexes on dotted keys
+            foo = Field(list)
+            bar = Field(dict)
+            __indexes__ = [
+                Index('bar.moo'),
+                # compund on two embedded document fields
+                Index([('bar.moo', pymongo.ASCENDING), ('bar.zoo', pymongo.ASCENDING)]),
+                # compound on embedded document field + list element document field
+                Index([('bar.moo', pymongo.ASCENDING), ('foo.whatever', pymongo.ASCENDING)]),
+            ]
+
+        Doc4.register(client=client, db='nanotestdb')
+        self.assertEqual(4, len(Doc4.get_collection().index_information()))  # 4 + _id
